import win32com.client
import sets
import minimax
from time import time

class Hexx:
    def __init__(self):
        # set initial position
        self.board = win32com.client.Dispatch("Python.Hexxagon")
        self.rtm = True                                     # red-to-move (true/false)
        self.rn = "ABCDEFG"                                 # row names

    def Reset(self):
        self.rtm = True
        self.board.Reset()

    def Move(self, move, draw = False):
        self.rtm = not self.rtm
        if move: self.board.Move(move, draw)                # move "None" means skip! (no possible moves for that player)

    def UndoMove(self, move, draw = False):
        self.rtm = not self.rtm
        if move: self.board.Undo(draw)

    def GetNeighbours(self, square):
        rx, dx = self.rn.index(square[0]), int(square[1])   # row and diagonal index
        neigh = []
        if rx > 0: neigh.append( self.rn[rx-1] + square[1] )
        if rx < 6: neigh.append( self.rn[rx+1] + square[1] )
        if dx > 1: neigh.append( square[0] + str(dx-1) )
        if dx < 7: neigh.append( square[0] + str(dx+1) )
        if rx > 0 and dx > 1: neigh.append( self.rn[rx-1] + str(dx-1) )
        if rx < 6 and dx < 7: neigh.append( self.rn[rx+1] + str(dx+1) )
        return neigh

    def GenerateMoves(self):
        if self.rtm:
            squares = self.board.Red()
        else:
            squares = self.board.White()
        empty = sets.Set(self.board.Empty())
        moves = []
        for sq in squares:
            growth = sets.Set(self.GetNeighbours(sq))
            neigh = sets.Set(growth)                    # first level neighbours
            for x in growth:
                neigh.update(self.GetNeighbours(x))     # second level neighbours
            neigh.intersection_update(empty)            # leave only empty neighbours
            moves.extend([sq + m for m in neigh])
        return moves

    def Evaluate(self):
        # the evaluation from the side-to-move's perspective (higher values are good for the side-to-move)
        return (len(self.board.Red()) - len(self.board.White())) * [-1,+1][self.rtm]
    
    
def Play(h, depth1, depth2):
    cx = 0
    rt = bt = 0
    while cx < 100:
        start = time()
        if h.rtm:
            alive = h.board.Red()
            move = minimax.GetMoveAB(h, depth1)
            rt += time() - start
        else:
            alive = h.board.White()
            move = minimax.GetMoveAB(h, depth2)
            bt += time() - start
        if not alive or not h.board.Empty():
            print "Moves made:", cx
            print "Red:", len(h.board.Red())
            print "White:", len(h.board.White())
            if len(h.board.Red()) > len(h.board.White()):
                print "Red won."
                print "Times:", rt, bt
            else:
                print "White won."
                print "Times:", rt, bt
            return
        h.Move(move, True)
        cx = cx + 1
    







    